/* Notes
    30 Jan 2014, part of remote link www.byvac.com
*/

// ===================================================================
// ===================================================================
// ===================================================================
// ===================================================================
// ADC Devices section
// ===================================================================
// ===================================================================
// A D C
// ===================================================================
// ADC
#define vrVDD  0
#define vr1024 0b10000001 // 1.024V
#define vr2048 0b10000010 // 2.048V
#define vr4096 0b10000011 // 4.096V
#define ITERATIONS 10

// ********************************************************************
// general purpose ADC conversion
// ********************************************************************
int adc(char vref, char chan, unsigned char aqu)
{
char loops;
long rv=0;
	if(vref == vrVDD) adcon1 = 0b11010000;
	else adcon1 = 0b11010011;
	fvrcon = vref;
	while(!(fvrcon & 0x40)); // wait for ref to be ready
	for(loops=0;loops < ITERATIONS; loops++) {
		adcon0 = (chan << 2) | 1; // select channel & turn on ADC
		delay_100us(aqu); // aqusition delay
		adcon0.GO = 1; // start conversion
		while(adcon0 & 2); // wait for conversion to complete
		rv += adresh << 8;
		rv += adresl;
	}
	adcon0 = 0; // trun off ADC to save power
	fvrcon = 0; // and fixed voltage ref
	return (int)(rv / ITERATIONS);
}

// ===================================================================
// ===================================================================
// L D R
// ===================================================================
// LDR
// settings
#define tLDR trisa.RA1
#define LDRCHAN 1
#define aLDR ansela.LDRCHAN

#ifdef TRANSMIT
// ********************************************************************
// ********************************************************************
void init_ldr()
{
	tLDR = 1; // input
	aLDR = 1; // analogue
}

// ********************************************************************
// this works for a reasonably low value LDR, about 50k dark value
// for higher or lower values, change the voltage refrence
// ********************************************************************
int ldr()	
{
	return adc(vr4096, LDRCHAN, 10);
}

#endif

// ===================================================================
// BATTERY VOLTAGE,
// ===================================================================
// BV
// settings
#define tBAT trisc.RC3
#define BATCHAN 7
#define aBAT anselc.ANSC3

#ifdef TRANSMIT
// ********************************************************************
// ********************************************************************
void init_bat()
{
	tBAT = 1; // input
	aBAT = 1; // analogue
}

// ********************************************************************
// this works for a reasonably low value LDR, about 50k dark value
// for higher or lower values, change the voltage refrence
// ********************************************************************
int bat()	
{
	return adc(vr2048, BATCHAN, 10);
}
#else
// ********************************************************************
// Battery voltage is converted to a voltage (x100) on recieve
// 100k + 47k gives 147/47 = 3.1277 multiplyer
// volts per step is 2.048 / 1023 = 0.002
// Voltage = (ADC *0.002) * 3.1277
// removing fp (ADC * 2 * 312 ) / 1000 gives to 2dp
// ********************************************************************
int get_bat(char *pkt)
{
int adc;
long rv;
	adc = *pkt * 256;
	adc += *(pkt+1) & 0xff;
	rv = adc * 2 * 3127;
	return(rv/10000);
}
#endif

// ===================================================================
// ===================================================================
// L M 3 5
// This has an output of 10mV per deg C ans so 25C will be 250mV
// Switch on time as per data sheet is 40 uS
// ===================================================================
// LM35 Temperature sensor
// settings
//#define tLM35 trisa.RA0
//#define oLM35 lata.RA0
//#define LM35CHAN 0
//#define aLM35 ansela.LM35CHAN

// ********************************************************************
// ********************************************************************
//void init_lm35()
//{
//	tLM35 = 1; // input
//	aLM35 = 1; // analogue
//}

// ********************************************************************
// expected results:
// 1.024 / 1023 = 0.999 per mV
// This is multipled by 1000 and so the result should be / 1000 to get
// 3 decimal places.
// ********************************************************************
//int lm35()	
//{
//	return (adc(vr1024, LM35CHAN, 10) * 99 ); // almost 1:1	
//}

// ===================================================================
// ===================================================================
// T M P 3 6
// This has an output of 10mV per deg C and is 750mV at 25 deg. C
// Switch on time as per data sheet is 40 uS
// ===================================================================
// TMP36 Temperature sensor, uses same pins as LM35
// settings
#define tTMP36 trisa.RA0
#define oTMP36 lata.RA0
#define TMP36CHAN 0
#define aTMP36 ansela.TMP36CHAN

#ifdef TRANSMIT
// ********************************************************************
// ********************************************************************
void init_tmp36()
{
	tTMP36 = 1; // input
	aTMP36 = 1; // analogue
}

// ********************************************************************
// ADC at 1.024V:
// 1.024 / 1023 = 0.999 per mV
// This device is calibrated to 750mV for 25
// 10mV per degree C
// 10mV below that is -1 deg.
// ********************************************************************
signed int tmp36()	
{
//return (adc(vr1024, TMP36CHAN, 10));
long t = (long)(adc(vr1024, TMP36CHAN, 10)) * 100;
	t -= 50000;
	return (signed int)(t/9); // to 2dp * 100
}
int tmp36raw()
{
	return adc(vr1024, TMP36CHAN, 10);
}
#endif
