// I2C
// Specific to this application
//
#define REMOVEOLD 3600 // time in seconds to remove an old packet
// macro *** Bit # from 0 to 7
#define bitset(var,bitno) ((var) |= 1UL << (bitno))
#define bitclr(var,bitno) ((var) &= ~(1UL << (bitno)))
// ******************************************************************
// Global I2C buffer
//unsigned char 	i2c_TIB[i2c_TIB_MAX+2];
unsigned char thePacket;
// ******************************************************************
// initialise i2c system but only if SDA (RC4) is high, this is held
// low by a resistor and only when it is connected to the I2C bus
// does it go high
// ******************************************************************
void i2c_init()
{
	// I2C lines so make sure they are i/p
	trisc.RC0=1; //SCL
	trisc.RC1=1; //SDA
	
	// address is in eeprom set up I2c
	// later
	ssp1add=I2CADDRESS; // set address
	sspcon=0x36;		// Setup SSP module for 7-bit, slave
	ssp1con2=1;			// SEN=1, enable clock stretching
	ssp1stat=0;			// just in case
	ssp1stat.CKE = 1;
	ssp1stat.SMP = 1;

	// Timer 2 with suitable pre and post delays
	// to give a 1 ms interrupt when pr2 is set to 128
	// i.e. 8us per value e.g. 10=80us
	t2con=0xfa;			// 0xf9=80us, 0xfa=320us
	t2con.2=0;			// off
	pr2=8;				// about 200us
	pie1.TMR2IE=1;		// enable interrupt

	// interrupts for i2c
	bitset(pie1,3);		// SSPIE Enable SPI interrupts
	pir1=0;				// clear all interrupt requests
	bitset(intcon,6);	// PEIE Enable all peripheral interrupts
	i2c_clear();		// clear o/p buffer

}

// ==================================================================
// 		I 2 C  O U T P U T  B U F F E R
// ==================================================================

// ******************************************************************
// clears buffer
// ******************************************************************
void i2c_clear()
{
	// reset
	bitset(sspcon,4);  // clear sclock stretch
}

// ******************************************************************
// i2c output buffer, this places a char on the bus
// ******************************************************************
void i2c_fill(unsigned char c)
{
	t2con.2=0;			// timer off
	pir1.TMR2IF=0;		// clear interrupt
//lcdHex(c);	
	while(sspcon & 0x10);  // wait until clock in stretch
	ssp1buf=c;
	delay_us(5);
	bitset(sspcon,4);  // clear sclock stretch
}

// ******************************************************************
// fail safe, is a user requests a read without any charatcers to
// read then this interrupt (after 200us) will return 0 to the i2c bus
// ******************************************************************
void i2c_isr_ccs() // clear clock stretch
{
//lcdData('x');
	t2con.2=0;			// timer off
	pir1.TMR2IF=0;		// clear interrupt
	ssp1buf=0;			// put 0 in bufffer
	bitset(sspcon,4);  // clear sclock stretch	
}

// ******************************************************************
// I2c handler called by interrupt (read,write from master point of view)
// uses serial buffer and if write then filles it, if read request
// then gets byte from o/p buffer and sends it
// ******************************************************************
void i2c_isr()
{
unsigned char c;
int j;
	c=ssp1buf;	// get recieved data
	if(ssp1stat & 0x20) { // 0 = address
		// data
		if(ssp1stat & 0x04) { // 1 = read
			// get data in o/p bufer and send
			pir1.TMR2IF=0;		// clear interrupt
			t2con.2=1;			// timer on			
		} else { // write
			// make a note of the packet required
			thePacket=c;
			bitset(sspcon,4);  // clear sclock stretch
		}
	}else{ // address
		if(ssp1stat & 0x04) { // read
			// get data in o/p buffer and send
			pir1.TMR2IF=0;		// clear interrupt
			t2con.2=1;			// timer on
			i2c_sv3(thePacket); // on read send all packets
		} else { 
			// if write juts throw away
			bitset(sspcon,4);  // clear sclock stretch
		}	
	}
	bitclr(sspcon,6);	// clear SSPOV if any
	bitclr(pir1,3);		// clear interrupt				
}

// ********************************************************************
// removes, sets contents to 0 of a particular packet number
// NOT USED - FUTURE
// ********************************************************************
void del_pkt(char pkt)
{
char j;
	for(j=1;j<PACKETS_MAX;j++) {
		if(packets[j].id == pkt) {
			packets[j].id = 0;
			packets[j].time = 0;
			packets[j].ia = 0;
			packets[j].ib = 0;
			packets[j].ic = 0;
			packets[j].volt = 0;
		}
	}
}

// ********************************************************************
// calculates packet age and stops when it gets to 65,535 seconds old
// also when the clock is reset back to 0 time can be greater then clock
// which causes problems wiht the calculation so this checks for that
// ********************************************************************
unsigned int get_age(char pkt)
{
unsigned long t;
unsigned int t1;
	// this happens at clock roll over
	if(packets[pkt].time > (clock/50000)) {
		packets[pkt].time = (clock/50000) - 1; // make sure smaller
		return 0;
	}
	t = (clock/50000)-packets[pkt].time; // time in seconds
	if(t > 0xffff) t1 = 0xffff;
	else t1 = (int)t;
	return t1;
}

// ********************************************************************
// removes, sets contents to 0 if the packet is over 120 minuits
// 7,200 seconds old
// ********************************************************************
void removeOld_pkt()
{
char j;
	for(j=1;j<PACKETS_MAX;j++) {
		if(get_age(j) > REMOVEOLD) {
			packets[j].id = 0;
			packets[j].time = 0;
			packets[j].ia = 0;
			packets[j].ib = 0;
			packets[j].ic = 0;
			packets[j].volt = 0;
		}
	}
}

// ********************************************************************
// ********************************************************************
void send_packet(char pkt)
{
unsigned int t1;
	i2c_fill(packets[pkt].id); // 1 byte
	i2c_fill(packets[pkt].ia >> 8); // high
	i2c_fill(packets[pkt].ia & 0xff); // low
	i2c_fill(packets[pkt].ib >> 8); // high
	i2c_fill(packets[pkt].ib & 0xff); // low
	i2c_fill(packets[pkt].ic >> 8); // high
	i2c_fill(packets[pkt].ic & 0xff); // low
	i2c_fill(packets[pkt].volt >> 8); // high
	i2c_fill(packets[pkt].volt & 0xff); // low
	t1 = get_age(pkt);
	i2c_fill(t1 >> 8); // high
	i2c_fill(t1 & 0xff); // low
}

// ======================================================================
// I2C SV3 Section, theses are equivalent to the SV3 serial commands
// and are provided to give a common set of commands for all devices.
// they are a combination of the old i2c commands and new ones
// all these commands are > 0x90
// ******************************************************************
//
// ******************************************************************
void i2c_sv3(unsigned char value)
{
	// clear packet available
	PKT=0;
	// command to clear all packets
	if(value == 0x80) {
		clear_packets(1); // all except system
		i2c_fill(0xaa);
		return;
	}
	if(value == 0x81) {
  		reset();
		return;
	}
	if(value > PACKETS_MAX) value = 0;
	send_packet(value); 
}
	
