// Remote Link Tanseceiver board RL03
// PCB Board RL03_0
//
// Default communication baud rate 115200
//
// RA0	(prog) TMP36
// RA1	(prog) LDR
// RA2	Q1
// RA3  Can be reset if needed - s/w configured
// RA4  LED
// RA5	SJ2 jumper any purpose, use weak pull up to use
// RC0	PKT input low sends packet (RX)
// RC1  Probe 2
// RC2  Probe 1
// RC3	Battery voltage
// RC4  connect Transmitter i/p (TX)
// RC5  Connect reciever o/p (RX) 
#define DEVICEID 9003 
#define VERSION  293  // v02.93

// ========== firmware type ==============
// define one of the following
//#define DEF_RX
#define DEF_RXA // *** SET for PIC16F1825
//#define DEF_TX
//#define DEF_TX_SW
//#define DEF_TX_PIR

// **** CHANGE CLOCK RATE TO 8000000 WHEN TX, 32000000 WHEN RX
//#define TRANSMIT  // rem for receive ***********************
//#define TRANSMIT_SW // pir version
//#define SUPERRX  // change device type to 1825 super receive

// Hardware DEFAULKTS
#define MINTXADR 0 // used in receive as well
#define MAXTXADR 15 // low nibble of ID

// RX
#ifdef DEF_RX
#define TXADDRESS 0 // low nibble TX id (default for RX)
#endif

// RXA
#ifdef DEF_RXA
#define SUPERRX
#define TXADDRESS 0 // low nibble TX id (default for RX)
#endif

// TX ADDRESS defaults
#ifdef DEF_TX
#define TXADDRESS 0 // low nibble TX id
#define TRANSMIT
#endif

#ifdef DEF_TX_SW
#define TXADDRESS 2 // low nibble TX id
#define TRANSMIT
#define TRANSMIT_SW
#endif

#ifdef DEF_TX_PIR
#define TXADDRESS 4 // low nibble TX id
#define TRANSMIT
#define TRANSMIT_SW
#endif


#ifdef SUPEREX
#undef DEVICEID
#define DEVICEID 9013
#endif


#ifdef TRANSMIT
#define _XTAL_FREQ 8000000 // 
#define EEADRLOC 0 // location of TXADDRESS

//for 1 wire
// global search state
#define MAX_DEVICES 2
unsigned char ROM_NO[8];
unsigned char OWd[MAX_DEVICES][8];

#else
#define _XTAL_FREQ 32000000
#endif


// data is a 16 bit integer so a paylod of 6 can handle 3 data items
// for example LDR,TMP36 and battery voltage
#define PAYLOAD 8 // in bytes, values are 2 bytes
#define PACKETSIZE (1+PAYLOAD+1)*3 // address+payload+csum
// timing values for setting frequency
// pulse periods x 10 us
#define ONEHIGH   120
#define ZEROHIGH  60
#define ENDOFBIT   50

// RX is RC5 
// TX is RC4
#define tTX trisc.RC4 // UART not used
#define TX latc.RC4 
#define tRX trisc.RC5
#define RX portc.RC5
// FET used to turn on or off sensor devices to save power
#define tQ1  trisa.RA2  // connected to FET
#define Q1	 lata.RA2
//
// configuration jumper
#define tSJ2	trisa.RA5 // defined also as packet indicator in rx
#define SJ2		porta.RA5
#define pSJ2	wpua.RA5
#define tLED	trisa.RA4
#define LED		lata.RA4 

#include <system.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <eeprom.h>

// ********************************************************************
// basic settings
// ********************************************************************
#pragma CLOCK_FREQ _XTAL_FREQ
// Set pic config word
// Config register 1
#pragma DATA _CONFIG1, _FOSC_INTOSC & _WDTE_SWDTEN & _PWRTE_ON & _MCLRE_OFF & _CP_ON & _CPD_OFF & _BOREN_OFF & _CLKOUTEN_OFF & _IESO_OFF & _FCMEN_OFF 
// Config register 2
#pragma DATA _CONFIG2, _WRT_OFF & _PLLEN_OFF & _STVREN_ON & _BORV_19 & _LVP_OFF
// eprom
#pragma DATA _EEPROM,TXADDRESS

// ********************************************************************
// Initialise osc
// ********************************************************************
#ifdef TRANSMIT
void init_oscillator (void)
{
	// Set oscillator frequency
	osccon.SPLLEN = 0; 	// PLL off
	osccon.IRCF3 = 1;
	osccon.IRCF2 = 1;
	osccon.IRCF1 = 1;
	osccon.IRCF0 = 0;	// 8MHz
}
#else
void init_oscillator (void)
{
	// Set oscillator frequency
	osccon.SPLLEN = 1; 	// PLL on
	osccon.IRCF3 = 1;
	osccon.IRCF2 = 1;
	osccon.IRCF1 = 1;
	osccon.IRCF0 = 0;	// 8MHz
}
#endif


// ********************************************************************
// ********************************************************************
void initHardware()
{

    // no ADC for now
	anselc = 0;
	ansela = 0;
	// weak pull ups
	wpua = 0; // disable
	wpuc = 0;
	option_reg.7 = 0; // enable global weak pull ups (WPUEN)
    // all i/p
	trisc = 0xff;
	trisa = 0xff;

    // Indicatro GREEN led L1
	tLED = 0; // output
	LED = 1; // 1 is off
	
	// optional solder jumper if used
	tSJ2 = 1; // input
	pSJ2 = 1; // enable weak pull up on jumper
	// fet used to turn devices on and off L2 (red)
	tQ1 = 0; // set as o/p
	Q1  = 0; // off by default

}

#include "devices.c"

#ifdef DEF_TX
#include "tx.c"
#endif

#ifdef DEF_TX_SW
#include "tx_sw.c"
#endif

#ifdef DEF_TX_PIR
#include "tx_pir.c"
#endif

#if defined(DEF_RX) || defined(SUPERRX)
#include "rx.c"
#endif






