// receiver module, decodes TX
// The existing PCB uses RC5 for the reception pin, so I am trying to fit
// in with this.
//
// MOP:
// FM signal (frequency) can be detected by the CCP input on every rising edge
//
// Pins
// no adc on RC5
#define tINP trisc.RC5
#define INP portc.RC5 // input from receiver
#define wINP wpuc.RC5 // weak pull up (global should also be enabled)
// 
// RC0 = SCL, RC1=SDA
// RC2,RC3,RA0,RA1,RA2 spare
#define tPKT	trisa.RA5 // indicates fresh packet
#define PKT		porta.RA5

//
#define I2CADDRESS  0x40

#include <system.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
//#include "serial.c" only for debug if required
#include "i2c.h"

// packet holds last packet recieved
typedef struct {
	unsigned char id; // id of tx
	unsigned int time; // in recieved time in seconds
	unsigned int ia;  // message a
	unsigned int ib; // message b
	unsigned int ic; // message c
	unsigned int volt; // batteryvoltage
} packet_t;
#ifdef SUPERRX
#define PACKETS_MAX  100  // packet 0 is system information (no room for 11)
#else
#define PACKETS_MAX  8
#endif

// ====================================================================
// RECEIVE SECTION
// ********************************************************************
// Global variables
// packetStore[] holds encoded data <address><payload><checksum>
//                                    3       3*4      3
// ********************************************************************
unsigned char packetStore[PACKETSIZE+2],packetAvailable=0;
unsigned long clock=0;
packet_t *packets;

// ********************************************************************
// clears packets, also used in i2c so placed here.
// ********************************************************************
void clear_packets(char n)
{
char j;
	// clear all packets
	for(j=n;j<PACKETS_MAX;j++) {
		packets[j].id = 0; // id of tx
		packets[j].time = 0; // in recieved time in seconds
		packets[j].ia = 0;  // message a
		packets[j].ib = 0; // message b
		packets[j].ic = 0; // message c
		packets[j].volt = 0;
	}
}

#include "i2c.c"

// ********************************************************************
// timer:
// Used to slice time into 20uS intervals, this is done by calling an
// interrupt
// ********************************************************************
void init_timer4()
{
	// clock is Fosc/4 = 8MHz = 125nS
	t4con= 0b00000010; // pre=16 gives 2uS timebase
	pir3.TMR4IF = 0; // clear flag
	pie3.TMR4IE = 1; // enable interrupt
	t4con.TMR4ON = 0; // timer off for now
    intcon.PEIE = 1; // timer needs perf interrupt enabled
	pr4 = 10;   // interrupt evey 20us
}
// timer 2 is used for i2c timeout, see i2c.c
//void init_timer2()
//{
//	// clock is Fosc/4 = 8MHz = 125nS
//	t2con= 0b01111011; // pre=64,post=16 gives 128uS timebase
//	pir1.TMR2IF = 0; // clear flag
//	pie1.TMR2IE = 1; // enable interrupt
//	t4con.TMR2ON = 0; // timer off for now
//    intcon.PEIE = 1; // timer needs perf interrupt enabled
//	pr2 = 255;   // interrupt evey 32ms
//}

// ********************************************************************
// This takes a slice of the input and counts the +ve slices, note
// that 1 and 0 are both +ve pulses
// The interrupt is called every 20uS which is about 160 instructions
// 1 = 1200uS
// 0 = 600uS
// Gap between bits is 500uS
// ********************************************************************
void interrupt()
{
static unsigned char startFound=0, thisByte=0, inpCount=0, packetCount=0,
					 bitCount=0, latched=0;
char theBit;

	if(pir1 & 8) i2c_isr(); // SSPIF (I2C)
	if(pir1.TMR2IF) i2c_isr_ccs(); 	// clears clock stretch on user error
	
	// this keeps track of when the packet was recieved, it is a 32bit
	// unsigned and so 1 sec = 50k, 1 min is 3000k and 1 hour is 
	// 180,000,000
	// so 4,294,967,295 / 180,000,000 = 23 hours
	clock++;
		
    // Count how many 20uS slices there have been when the INP is +ve
    // When INP is 0, the count is ignored because latch is set to 0
    if(INP) {
        inpCount++;
        if(inpCount > 2) latched = 1;
    } else {
        if(latched) { // The H-L transition latches the bit
			// start sync is 2 consecutive bits 3mS long
			if(startFound < 2) {
				if((inpCount > 120) && (inpCount < 160)) startFound++;
				else startFound = 0;

				bitCount=0; // for when start found
				inpCount = 0;
				latched = 0;
			} else {
				// count determins if the bit is 1 or 0. This calculation will
				// effect the whole performance
				//LED=1; //Don't turn this on in here
				theBit =  2; // assume no good
				if((inpCount > 22) && (inpCount < 37)) theBit = 0;
				if((inpCount > 40) && (inpCount < 74)) theBit = 1;
				inpCount = 0;
				latched = 0;

				// compile bytes into packet
				if(theBit < 2) {
					if(packetCount >= PACKETSIZE) { // done
						packetAvailable = 1;
						startFound = 0; // for next time
						packetCount = 0; // for next time
						t4con.TMR4ON = 0; // timer off
					} else {
						// assemble byte
						thisByte <<= 1;
						thisByte += theBit;
						if(bitCount++ > 6) { // 6 because of post increment
							packetStore[packetCount++] = thisByte;
							bitCount=0;
						}
					}
				}
			} // startFound
		}// latched
	}
    pir3.TMR4IF = 0; // clear interrupt
}

// ********************************************************************
// byte decode, given three encoded bytes it will decode into a single
// byte with error correction.
// 1 = '111'
// 0 = '000'
// Hamming correction via parity
// ********************************************************************
unsigned char byte_decode(unsigned char msg[])
{
long coded;
unsigned char b=0, triptik, bits=8, codeshift = 21;
	coded = ((long)msg[0]) << 16; //pnum("\rmsg0%X",msg[0]);
	coded |= ((long)msg[1]) << 8; //pnum(" msg1%X",msg[1]);
	coded |= ((long)msg[2]) & 0xff;  //pnum(" msg2%X",msg[2]);
//pnum("\ncoded=%X",coded);	
	while(bits) {
		triptik = (coded  >> codeshift) & 7;
//	pnum("\rtrip=%02X",triptik);	
		//if(triptik == 7) b += 1; // no correction
		if((triptik == 3) || (triptik == 5) || (triptik == 6) || (triptik == 7)) b += 1;
		//if(triptik == 0) b &= ~bytemask; // no correction
		//if((triptik== 0) || (triptik == 1) || (triptik == 2) || (triptik == 4)) b &= ~mask;
		bits--;
		codeshift -= 3;
		if(bits) b <<= 1;
	}
//	pnum("\rb=%02X",b);
	return b;
}

// ********************************************************************
// ********************************************************************
//void display_packets()
//{
//char j;
//	for(j=0;j<PACKETS_MAX;j++) {
//		pnum("%u,",packets[j].id);
//		pnum("%u,",packets[j].ia);
//		pnum("%d,",packets[j].ib);
//		pnum("%d,",packets[j].volt);
//		pnum("%u",(clock/50000)-packets[j].time); // time in seconds
//		puts("\r");
//	}
//}
// ********************************************************************
// decodes 2 consecutive bytes h,l
// ********************************************************************
int get_16(char *pkt)
{
int rv;
	rv = *pkt * 256;
	rv += *(pkt+1) & 0xff;
	return rv;
}

// ********************************************************************
// ********************************************************************
void main(void)
{
char j,k,nclock=0;

	// system initialisation
   	init_oscillator ();
	delay_ms(50);
	init_timer4();
	
	initHardware();
    // ports and switches
	tPKT= 0; // indicates freah packet available
	PKT = 0;
	tINP = 1; // main rx input
	wINP = 1; // enable weak pull up    

	// sign on
    for(j=0;j<5;j++) {
	   LED = 1; delay_ms(40);
	   LED = 0; delay_ms(40);
    }
    LED = 1; // off
    Q1 = 1; // Mosfet on, provides power to RX
	
	intcon.GIE = 1; // Global

//	init_serial(_XTAL_FREQ,9600);
	i2c_init();
//	puts("Startup...");

	packets = (packet_t*)alloc(sizeof(packet_t)*PACKETS_MAX);
	
	// ERROR check if wrong chip has been programmed
	if(!packets) {
		while(1) {
			LED=0;
			delay_s(1);
			LED=1;
			delay_s(1);
		}
	}

	clear_packets(0); // clear all packets
	
	// add receiver information
	// <packet ID><device ID><sw version>
	packets[0].id = 10; // packet id for system information
	packets[0].ia = DEVICEID;  // device id
	packets[0].ib = VERSION; // sw version
	packets[0].ic = 0; // number of bad packets
	packets[0].volt = 0; // not used
	packets[0].time = 0; // will indicate latest packet


	// clear clock
	clock=0;

	while(1) {
		unsigned char adr, csum=0, decoded[PAYLOAD+3],k;
		packetAvailable = 0;
		t4con.TMR4ON = 1; // turn on timer - truned off when pkt recieved
		while(!packetAvailable); // wait for packet

		// unsigned long van cause problems after a certain size do keep this down
		// to say 2 hours (unsigned long should go to 23 hours). NOTE the 
		// problem was probably with the age calculation but there is no harm in
		// keeping the count to a reasonable figure
		// 28800 is 4 hours in seconds
		if(clock > (50000 * 28800)) clock = 0;

		// packetStore[] holds encoded data <address><payload><checksum>
		//                                    3       3*4      3
		// Decode received packet
		adr = byte_decode(&packetStore[0]); //0-2
		csum += adr;
		k = 3;
		for(j=0;j<PAYLOAD;j++) {
			decoded[j] = byte_decode(&packetStore[k]);
			csum += decoded[j];
			k += 3;
		}
		csum += byte_decode(&packetStore[3+(PAYLOAD*3)]);

		if(!csum) {
			// add or update this ID into the packets store
			LED=0;  // on
			PKT = 1; // packet is available

			// packet must be in range 
			//if((adr & 0xf) > MAXTXADR) continue; // leave LED on
			if(adr == 0) continue;
						
			nclock=0; // reset, reset bad packet timer
			
			// system time set to when last packet ID received
			packets[0].time = (int)(clock/50000);
			packets[0].volt = adr; // which packet
			

			for(j=1;j<PACKETS_MAX;j++) {
				if(packets[j].id == 0) { // new packet
					packets[j].id = adr;
					packets[j].time = (int)(clock/50000);
					packets[j].ia = get_16(&decoded[0]);
					packets[j].ib = get_16(&decoded[2]);
					packets[j].ic = get_16(&decoded[4]);
					packets[j].volt = get_bat(&decoded[6]);		
					break;
				} else {
					if(packets[j].id == adr) {
					packets[j].time = (int)(clock/50000);
					packets[j].ia = get_16(&decoded[0]);
					packets[j].ib = get_16(&decoded[2]);
					packets[j].ic = get_16(&decoded[4]);
					packets[j].volt = get_bat(&decoded[6]);
					break;
					}
				}
			}
			
			// addition to 1.82, romoves packest older than 2 hours
			removeOld_pkt();
		} else {
			// no check sum, allow 10 bad packets before reset
			nclock++;
			packets[0].ic = nclock;
		}
		LED=1; // off
		if(nclock > 20) reset();
	}
}