// transmit module, special bit centric code. 
// Original signal coding from 2.4


// watchdog timings at 8mHz
// #define interval 0b00010110 // 2S
// #define interval 0b00011000 // 4S
// #define interval 0b00011010 // 8S
// #define interval 0b00011100 // 16S
// #define interval 0b00011110 // 32S
#define interval 0b00100000 // 64S
// #define interval 0b00100010 // 128S
// #define interval 0b00100100 // 256S

#include "ds1820.c"
#include "an187.c"
#include "asong1wire.c"
#include "serial.c"

// ********************************************************************
// sends a given byte onto the TX pin
// *******************************************************************
void send_byte(char b)
{
unsigned char mask=0x80;
	while(1) {
		if(b & mask) {
            TX=1; delay_10us(ONEHIGH);
        }
		else {
            TX=1; delay_10us(ZEROHIGH);
        }
        TX=0; delay_10us(ENDOFBIT);
		if(!(mask>>=1)) break;
	}
}

// *****************************************************************************
// convert byte for 3 bit hamming code and send, each byte is 24 bits
// 0 is 000 and 1 is 111
// send msb first
// *****************************************************************************
void send_ham_byte(char data)
{
long coded=0;
char bytemask=0x80;
	while(bytemask){
		if(data & bytemask) coded |= 7;
		if(bytemask==1) break; // no need for last push
		coded <<= 3;
		bytemask >>= 1;
	}
	send_byte((coded >> 16) & 0xff);
	send_byte((coded >> 8) & 0xff);
	send_byte(coded & 0xff);
}

// ********************************************************************
// sends
// <preabmle><sync-start><address><payload><checksum>
// ********************************************************************
void send_message(char *pl, char address)
{
unsigned char csum=0, j, *cp=pl;
    // preabmble. This gets the RX going and has to be this long
    // for better results. The atcual time spent pulsing is important
    // the actual width of the pulse is not.
    // This takes 40mS.
    // Relatively long (1mS pulses) appear to give the best results
   for(j=0;j<20;j++) {
		delay_10us(100);
		TX=1;
		delay_10us(100);
		TX=0;
	}
	// Sync condition - detect two consecutive long pulses
	//LED = 1;
	delay_10us(100);
	TX=1; delay_ms(3);
	TX=0; delay_10us(ENDOFBIT);
	TX=1; delay_ms(3);
	TX=0; delay_10us(ENDOFBIT);

	// device addess
    csum += address;
    send_ham_byte(address); //
    //LED=0;

	// packet
    for(j=0;j<PAYLOAD;j++) {
		csum += *cp;
		send_ham_byte(*(cp++));
	}

	// using unsigned char, adding this to the end reult will be 0
	// this is the same type of csum used for hex files, simple
	// but can give a false positive
	send_ham_byte(0-csum);
}

// *****************************************************************************
// 
// *****************************************************************************
void main(void)
{
unsigned char msg[PAYLOAD], j, rdel, owd=0, devID=0x10;
signed int d1;
unsigned long d2;
char first = 0; // first time send a few packets
	// system initialisation
   	init_oscillator();
	delay_ms(50);

	initHardware();
	intcon.GIE = 0; // Global	

	tTX = 0; // output pin to transmitter
	tQ1 = 0; // FET controlling devices (LED2 is conneted to this)
	Q1 = 0; // off

	// WATCHDOG USED FOR TIMING with differnt delay each time
	// clash only if 2 TX started at exactly same time
	rdel = 1;
	
	// sign on	
    for(j=0;j<5;j++) {
	   LED = 1; delay_ms(250);
	   LED = 0; delay_ms(100);
    }
    LED = 0; // on


    // select address if jumper sJ2 is 0 and store that in EEPROM
    if(!SJ2) {
		j = eeprom_read(EEADRLOC) + 1;
		if(j > MAXTXADR) 
			j = MINTXADR;
		eeprom_write(EEADRLOC,j);
		while(!SJ2); // wait for release
		// indicate the address by using flashes
		while(j > MINTXADR) {
			LED = 0; // on
			delay_ms(450);
			LED = 1;
			delay_ms(250);
			j--;
		}
	}
	
	wdtcon = interval; // main interval
	wdtcon.SWDTEN = 0; // watchdog off just in case
	Q1 = 1; // turn on devices

// ADD device initialisation here
	init_WW(); // am2302 1 wire device 
	ow_init();
	init_ldr();
	init_tmp36();
	init_bat(); // battery voltage monitor

	// The high nibble for the device ID is as follows:
	// see tx_sw.c	
	delay_s(2); // for AM2302 to warm up if its here
	
	if(WW_read())  {
		devID = 0x60; // can only be this
	} else {
		devID = 0x10; // assume tmp36
		owd = OWfillRoms(); // fill fom and get number of devices
		if(owd == 1) devID = 0x20; // 1-wire
		if(owd == 2) devID = 0x30; // 2 x 1-wire
	}
		
	// add actual device address to ID
	devID += eeprom_read(EEADRLOC);
	rdel = eeprom_read(EEADRLOC);
	if((devID & 0xf0) != 0x60) off_WW(); // turn off timer to save power
	adcon0 = 0; // thish shuld also be off, here just in case

	Q1 = 0; // device now off
	LED = 1; // off not used anymore

	for(;;) {			
		Q1 = 1; // turn on devices
		wdtcon.SWDTEN = 0; // watchdog off
		if((devID & 0xf0) == 0x60) delay_s(2); // AM2302 needs this
		else delay_ms(2); // needs for battery cap to charge 10nf
		
        // LDR		AMtemp	AMhum
        // clear makes things nice and tidy
        for(j=0;j<PAYLOAD;j++) msg[j]=0;
        
        // LDR is always first packet
        d1 = ldr();
		msg[0]=(d1>>8);
		msg[1]=d1 & 0xff;    
		
		// TMP32    
		if((devID & 0xf0) == 0x10) {
			d1 = tmp36raw();
			msg[2]=(d1>>8);
			msg[3]=d1 & 0xff;    
		}
		
		// 1wire (1)
		if((devID & 0xf0) == 0x20) {
			d1 = ow_temp(0);
			msg[2]=(d1>>8);
			msg[3]=d1 & 0xff;    
		}

		// 1wire (2)
		if((devID & 0xf0) == 0x30) {
			d1 = ow_temp(0);
			msg[2]=(d1>>8);
			msg[3]=d1 & 0xff;   
			d1 = ow_temp(1);
			msg[4]=(d1>>8);
			msg[5]=d1 & 0xff;    
		}

		// AM2302 
        if((devID & 0xf0) == 0x60) {
			d2 = WW_read(); // 32 bit integer
			d1 = (d2 & 0xffff); // temperature
			msg[2]=(d1>>8);
			msg[3]=d1 & 0xff;        
			d1 = ((d2 >> 16)& 0xffff); // humidity
			msg[4]=(d1>>8);
			msg[5]=d1 & 0xff;        
		}

		// battery
		d1 = bat();
		msg[6]=(d1>>8);
		msg[7]=d1 & 0xff;

		// transmit information
		send_message(msg, devID);
		// turn off devices
		Q1 = 0; // turn off devices
		
		// first time on feature
		if(first++ < 10) {
			delay_ms(200);
			continue;
		}
		
		// restart scycle after delay set by device ID
		delay_s(rdel);
		
		wdtcon.SWDTEN = 1; // watchdog on
		//LED=0;
		sleep();
	}
	
}


