// transmit module, special bit centric code. 
// Original signal coding from 2.4


// *********** ERROR on RL03_SW_a **************************************
// 18 Dec 2015
// Correction:
// Connect RA4 to RC0 on the actual chip and make sure that RC0 is an input
// coding done but can't test until board arrives.
//
// watchdog timings at 8mHz
// #define interval 0b00010110 // 2S
// #define interval 0b00011000 // 4S
// #define interval 0b00011010 // 8S
// #define interval 0b00011100 // 16S
// #define interval 0b00011110 // 32S
#define interval 0b00100000 // 64S
// #define interval 0b00100010 // 128S
// #define interval 0b00100100 // 256S

#include "ds1820.c"
#include "an187.c"
#include "asong1wire.c"
#include "serial.c"

// this is for the PIR detector
#define PIR	porta.RA4
#define aPIR ansela.RA4
#define uPIR wpua.RA4 // weak pull up GLOBAL enabled in main
#define tPIR trisa.RA4
#define cPIR iocap.RA4 // on change positive
#define nPIR iocan.RA4 // change on negative
#define fPIR iocaf.RA4 // on change flag

// ********************************************************************
// sends a given byte onto the TX pin
// *******************************************************************
void send_byte(char b)
{
unsigned char mask=0x80;
	while(1) {
		if(b & mask) {
            TX=1; delay_10us(ONEHIGH);
        }
		else {
            TX=1; delay_10us(ZEROHIGH);
        }
        TX=0; delay_10us(ENDOFBIT);
		if(!(mask>>=1)) break;
	}
}

// *****************************************************************************
// convert byte for 3 bit hamming code and send, each byte is 24 bits
// 0 is 000 and 1 is 111
// send msb first
// *****************************************************************************
void send_ham_byte(char data)
{
long coded=0;
char bytemask=0x80;
	while(bytemask){
		if(data & bytemask) coded |= 7;
		if(bytemask==1) break; // no need for last push
		coded <<= 3;
		bytemask >>= 1;
	}
	send_byte((coded >> 16) & 0xff);
	send_byte((coded >> 8) & 0xff);
	send_byte(coded & 0xff);
}

// ********************************************************************
// sends
// <preabmle><sync-start><address><payload><checksum>
// ********************************************************************
void send_message(char *pl, char address)
{
unsigned char csum=0, j, *cp=pl;
    // preabmble. This gets the RX going and has to be this long
    // for better results. The atcual time spent pulsing is important
    // the actual width of the pulse is not.
    // This takes 40mS.
    // Relatively long (1mS pulses) appear to give the best results
   for(j=0;j<20;j++) {
		delay_10us(100);
		TX=1;
		delay_10us(100);
		TX=0;
	}
	// Sync condition - detect two consecutive long pulses
	//LED = 1;
	delay_10us(100);
	TX=1; delay_ms(3);
	TX=0; delay_10us(ENDOFBIT);
	TX=1; delay_ms(3);
	TX=0; delay_10us(ENDOFBIT);

	// device addess
    csum += address;
    send_ham_byte(address); //
    //LED=0;

	// packet
    for(j=0;j<PAYLOAD;j++) {
		csum += *cp;
		send_ham_byte(*(cp++));
	}

	// using unsigned char, adding this to the end reult will be 0
	// this is the same type of csum used for hex files, simple
	// but can give a false positive
	send_ham_byte(0-csum);
}

// *****************************************************************************
// 
// *****************************************************************************
void main(void)
{
unsigned char msg[PAYLOAD], j, rdel, owd=0, devID=0x10;
signed int d1;
unsigned long d2;
char first = 0; // first time send a few packets
	// system initialisation
   	init_oscillator();
	delay_ms(50);

	// initialsie its own hardware to save power
	// no ADC for now
	anselc = 0;
	ansela = 0;
	// weak pull ups for config pads
	wpua = 0; // disable
	wpuc = 0;
	option_reg.7 = 0; // enable global weak pull ups (WPUEN)
    // all i/p
	trisc = 0xff;
	trisa = 0xff;
	// config pad
	tSJ2 = 1; // input
	pSJ2 = 1; // enable weak pull up on jumper
	// fet used to turn devices on and off L2 (red)
	tQ1 = 0; // set as o/p
	Q1  = 0; // off by default
		
	tTX = 0; // output pin to transmitter

	// WATCHDOG USED FOR TIMING with differnt delay each time
	// clash only if 2 TX started at exactly same time
	rdel = 1;	

    // select address if jumper sJ2 is 0 and store that in EEPROM
    if(!SJ2) {
		j = eeprom_read(EEADRLOC) + 1;
		if(j > MAXTXADR) 
			j = MINTXADR;
		eeprom_write(EEADRLOC,j);
		while(!SJ2); // wait for release
		// indicate the address by using flashes
		while(j > MINTXADR) {
			Q1 = 0; // on
			delay_ms(450);
			Q1 = 1;
			delay_ms(250);
			j--;
		}
	}
	
	wdtcon = interval; // main interval
	wdtcon.SWDTEN = 0; // watchdog off just in case
	
	// 2 second warm up time no pir connected
	Q1=1;
	for(j=0;j<2;j++) {
		delay_s(1);
	}

	// ADD device initialisation here
	init_WW(); // am2302 1 wire device 
	init_ldr();
	ow_init(); // 1-wire device is better (cheaper)
	init_tmp36();	
	init_bat(); // battery voltage monitor

	// SW version has PIR connected. If no PIR exists then the input
	// needs connecting to ground
	aPIR = 0; // digital
	tPIR = 1; // input
	//uPIR = 1; // pull up consumes power when connected to ground 170uA
	cPIR = 1; // change on positive
	nPIR = 1; // change on negative
	fPIR = 0; // clear flag
	intcon.PEIE = 1; // needed
	intcon.IOCIE = 1; // global interrupt on change
	intcon.GIE=0; // no jump to interrupt
	
	// ********** RL03_sw_a hardware correction *****************
	trisc.RC0 = 1; // RA4 is normally LED
		
	if(WW_read())  {
		devID = 0x60; // can only be this
		intcon.IOCIE = 0; // turn off PIR global interrupt on change
	} else {
		ow_init(); // just in case
		owd = OWfillRoms(); // fill fom and get number of devices
		if(owd == 1) devID = 0x70; // 1-wire + pir
		else {
			devID = 0x80; // tmp36 + PIR
		}
	}

// The high nibble for the device ID is as follows:
// 1 - tmp36
// 2 - 1-wire
// 3 - 2 x 1-wire
// 4 - 1-wire + PIR
// 5 - tmp36 + PIR
// 6 - DHT
// 7 - 1-wire + SW
// 8 - 1 tmp36 + SW
	
	// add actual device address to ID
	devID += eeprom_read(EEADRLOC);
	rdel = eeprom_read(EEADRLOC);
	if((devID & 0xf0) != 0x60) off_WW(); // turn off timer to save power
	adcon0 = 0; // thish shuld also be off, here just in case

// ========================== debug section =======================
//init_serial(_XTAL_FREQ,115200);
//puts("Startup...");	
//pnum("\ntemp0=%d\n",ow_temp(0));
//pnum("\ntemp1=%d\n",ow_temp(1));
//pnum("\ntemp1=%X\n",devID);
//ow_init();
//owd = OWfillRoms();
//sprintf32(msg,"\n%d",owd);
//puts(msg);
//	puts("\n");
//	for(j=0;j<8;j++)
//		pnum("%X ",OWd[0][j]);
//	pnum("  %d",ow_temp(0));
//	puts("\n");
//	for(j=0;j<8;j++)
//		pnum("%X ",OWd[1][j]);
//	pnum("  %d",ow_temp(1));
//puts("\nDone...");
//while(1);			
// =================================================================

	Q1 = 0; // device now off
	PIR; // read port and clear flag
	fPIR = 0; // clear flag

	for(;;) {			
		// some changes for this device but it still sends every
		// so often
		// **** CHANGE CLOCK RATE TO 8000000 to use delay_x()
		Q1 = 1; // turn on devices
		wdtcon.SWDTEN = 0; // watchdog off		
		
		// 2s needed for DHT
		if((devID & 0xf0) == 0x60) delay_s(2);
		else delay_ms(2); // needs for battery cap to charge 10nf

        // clear makes things nice and tidy
        for(j=0;j<PAYLOAD;j++) msg[j]=0;
        
        // LDR is always first packet
        d1 = ldr();
		msg[0]=(d1>>8);
		msg[1]=d1 & 0xff;    
		
		// TMP36    
		if((devID & 0xf0) == 0x80) {
			d1 = tmp36raw();
			msg[2]=(d1>>8);
			msg[3]=d1 & 0xff;    
		}
		
		// 1-wire
		if((devID & 0xf0) == 0x70) {
			d1 = ow_temp(0);
			msg[2]=(d1>>8);
			msg[3]=d1 & 0xff;    
		}
		
		if((devID & 0xf0) == 0x60) {
			on_WW(); // timer on
			d2 = WW_read(); // 32 bit integer
			d1 = (d2 & 0xffff); // temperature
			msg[2]=(d1>>8);
			msg[3]=d1 & 0xff;        
			d1 = ((d2 >> 16)& 0xffff); // humidity
			msg[4]=(d1>>8);
			msg[5]=d1 & 0xff;        
			off_WW(); // timer off to save power
		} else {
			// Switch uses PIR pads
			d1 = 0;
			if(PIR) {
				fPIR = 0; // clear flag				
				//while(PIR){ delay_ms(1); d1++; } // wait
				delay_ms(100); // debounce
				d1 = PIR;
			}
			msg[4]=(d1>>8);
			msg[5]=d1 & 0xff;    
		}

		// battery
		d1 = bat();
		msg[6]=(d1>>8);
		msg[7]=d1 & 0xff;

		// transmit information
		send_message(msg, devID);
		// turn off devices
		Q1 = 0; // turn off devices
		
		// restart scycle after random delay
		delay_s(rdel);

		PIR; // read port and clear flag
		fPIR = 0; // clear flag

		wdtcon.SWDTEN = 1; // watchdog on
		sleep();
	}
	
}


